<?xml version="1.0"?>
<xsl:stylesheet version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xlink="http://www.w3.org/1999/xlink"
	xmlns:xtm="http://www.topicmaps.org/xtm/1.0/">
<xsl:output encoding='iso-8859-1' indent='yes'/>

<xsl:key name='topic' match='xtm:topic' use='concat("#",@id)'/>
<xsl:key name='baseName' match='xtm:baseNameString'
	use='ancestor::xtm:baseName/ancestor::xtm:topic'/>

<xsl:template match="/xtm:topicMap">
<TOUCHGRAPH_LB version="1.20">
    <NODESET>	
		<NODE nodeID="tt-unknown">
			<NODE_LOCATION visible="true" />
		<NODE_LABEL label="Unknown Type" shape="1" backColor="333333" textColor="FFFFFF" fontSize="12" />
		</NODE>
		<xsl:apply-templates select='xtm:topic' mode='nodes'/>
		<xsl:apply-templates select='xtm:association' mode='nodes'/>
</NODESET>

    <EDGESET>
    	<xsl:apply-templates select='xtm:topic' mode='edges'/>
		<xsl:apply-templates select='xtm:association' mode='edges'/>
	</EDGESET>
</TOUCHGRAPH_LB>
</xsl:template>

<xsl:template match='xtm:topic' mode='nodes'>
	<xsl:variable name='name' select='key("baseName",.)'/>
	<NODE nodeID='{@id}'>
		<NODE_LOCATION visible="false"/>
		<NODE_LABEL label="{$name}" shape="1" 
			backColor="00A0F0" textColor="FFFFFF" fontSize="12"/>
	</NODE>

</xsl:template>

<xsl:template match='xtm:topic' mode='edges'>
	<xsl:variable name='type' 
		select='substring-after(xtm:instanceOf/xtm:topicRef/@xlink:href,"#")'/>
	<xsl:variable name='name' select='key("baseName",.)'/>
	
	<!-- Check to see if the target of this edge actually exists-->
	<xsl:variable name='islegal' select='count(key("topic",concat("#",$type)))>0'/>
	
	<xsl:if test='$islegal or $type=""'>
		<xsl:variable name='realtype'
			><xsl:choose>
				<xsl:when test='$type!=""'><xsl:value-of select='$type'/></xsl:when>
				<xsl:otherwise>tt-unknown</xsl:otherwise>
			</xsl:choose
		></xsl:variable>
		<EDGE fromID="{@id}" toID="{$realtype}" type="1" 
			length="7000" visible="true" color="A0A0A0"/>
	</xsl:if>
</xsl:template>

<xsl:template match='xtm:association' mode='nodes'>
	<xsl:variable name='realid'>
		<xsl:call-template name='realid'>
			<xsl:with-param name='id' select='@id'/>
		</xsl:call-template>
	</xsl:variable>
	
	<xsl:variable name='typename'
		><xsl:call-template name='typename'
	/></xsl:variable>
	<NODE nodeID='{$realid}'>
		<NODE_LOCATION visible="false"/>
		<NODE_LABEL label="{$typename}" shape="3" 
			backColor="00A0F0" textColor="FFFFFF" fontSize="12"/>
	</NODE>

</xsl:template>

<xsl:template match='xtm:association' mode='edges'>
	<xsl:variable name='realid'>
		<xsl:call-template name='realid'
			><xsl:with-param name='id' select='@id'
		/></xsl:call-template
	></xsl:variable>
	
	<!-- Check to see if the target of this edge actually exists-->
	<xsl:variable name='type' select='xtm:instanceOf/xtm:topicRef/@xlink:href'/>
	<xsl:variable name='islegal' 
		select='count(key("topic",$type))>0'/>
	
	<xsl:if test='$islegal or $type=""'>
		<!--xsl:name='realtype'>
			<xsl:call-template name='realid'>
				<xsl:with-param name='id' select='@id'/>
			</xsl:call-template>
		</xsl:variable-->
		<xsl:variable name='realtype'><xsl:call-template name='realtype'
		/></xsl:variable>
	
		<!-- Edge to the association type -->
		<EDGE fromID="{$realid}" toID="{$realtype}" type="1" 
			length="7000" visible="true" color="A0A0A0"/>
	</xsl:if>
	
	<!-- Edges to role-playing topics -->
	<xsl:apply-templates select='xtm:member'>
		<xsl:with-param name='associd' select='$realid'/>
	</xsl:apply-templates>
</xsl:template>


<xsl:template match='xtm:member'>
	<xsl:param name='associd'/>
	<xsl:apply-templates select='xtm:topicRef' mode='player'>
		<xsl:with-param name='associd' select='$associd'/>
	</xsl:apply-templates>
</xsl:template>

<xsl:template match='xtm:topicRef' mode='player'>
	<xsl:param name='associd'/>
	<xsl:variable name='to' select='substring-after(@xlink:href,"#")'/>
	
	<!-- Check to see if there is a topic with the "to" id -->
	<xsl:variable name='islegal' select='count(key("topic",concat("#",$to)))>0'/>
	<xsl:if test='$islegal>0'>
		<EDGE fromID="{$associd}" toID='{$to}' type="1" 
			length="7000" visible="true" color="000000"/>	
	</xsl:if>
</xsl:template>

<xsl:template name='getname'>
<!-- enter with a topic as the context -->
	<xsl:value-of select='normalize-space(xtm:baseName/xtm:baseNameString)'/>
</xsl:template>

<!-- Create id value when it is missing -->
<xsl:template name='realid'>
	<xsl:param name='id'/>
	<xsl:choose>
		<xsl:when test='@id!=""'><xsl:value-of select='@id'/></xsl:when>
		<xsl:otherwise><xsl:value-of select='generate-id(.)'/></xsl:otherwise>
	</xsl:choose>
</xsl:template>

<!-- Get type, using "tt-unknown" if missing -->
<!-- Enter with an object that has an instanceOf -->
<xsl:template name='realtype'>
	<xsl:variable name='id' 
		select='substring-after(xtm:instanceOf/xtm:topicRef/@xlink:href,"#")'/>
	<xsl:choose>
		<xsl:when test='$id!=""'><xsl:value-of select='$id'/></xsl:when>
		<xsl:otherwise><xsl:value-of select='generate-id(.)'/></xsl:otherwise>
	</xsl:choose>
</xsl:template>


<!-- Get the basename string of the type of an element -->
<xsl:template name='xtypename'>
	<xsl:variable name='type' 
		select='substring-after(xtm:instanceOf/xtm:topicRef/@xlink:href,"#")'
		/><xsl:if test='$type!=""'>
			<xsl:value-of select='/xtm:topicMap/xtm:topic[@id=$type]
				/xtm:baseName/xtm:baseNameString'
		/></xsl:if>
</xsl:template>

<xsl:template name='typename'>
	<xsl:variable name='typeid' 
		select='xtm:instanceOf/xtm:topicRef/@xlink:href'
		/><xsl:if test='$typeid!=""'>
			<xsl:value-of select='key("topic",$typeid)
				/xtm:baseName/xtm:baseNameString'
		/></xsl:if>
</xsl:template>

</xsl:stylesheet>
